@@ -76,7 +76,6 @@ func TestWebhook(t *testing.T) {
 
 		for _, tt := range hookHandlerTests {
 			t.Run(tt.desc, func(t *testing.T) {
-
 				ip, port := serverAddress(t)
 				args := []string{fmt.Sprintf("-hooks=%s", configPath), fmt.Sprintf("-ip=%s", ip), fmt.Sprintf("-port=%s", port), "-verbose"}
 
@@ -249,7 +248,7 @@ func serverAddress(t *testing.T) (string, string) {
 func waitForServerReady(t *testing.T, ip, port string) {
 	waitForServer(t,
 		fmt.Sprintf("http://%v:%v/", ip, port),
-		http.StatusNotFound,
+		http.StatusOK,
 		5*time.Second)
 }
 
@@ -618,6 +617,7 @@ env: HOOK_head_commit.timestamp=2013-03-12T08:14:29-07:00
 	// Check logs
 	{"static params should pass", "static-params-ok", nil, `{}`, false, http.StatusOK, "arg: passed\n", `(?s)command output: arg: passed`},
 	{"command with space logs warning", "warn-on-space", nil, `{}`, false, http.StatusInternalServerError, "Error occurred while executing the hook's command. Please check your logs for more details.", `(?s)unable to locate command.*use 'pass[-]arguments[-]to[-]command' to specify args`},
+	{"unsupported content type error", "github", map[string]string{"Content-Type": "nonexistent/format"}, `{}`, false, http.StatusBadRequest, `Hook rules were not satisfied.`, `(?s)error parsing body payload due to unsupported content type header:`},
 }
 
 // buffer provides a concurrency-safe bytes.Buffer to tests above.
